#Series of text parsing functions to derive self-reported sleep patterns from the EHR
#Input: ClinDocs
#Output: SleepDataRaw.csv
import pandas as pd
import re, gc
import numpy as np
from itertools import chain
from pathos.helpers import mp

def ChronoEstimator(MedRecords):
    VisitYear = re.findall('\*DATE.*(20\d\d)|\*DATE.*(199\d)', MedRecords) #updated - assume either 1990s or since 2000
    VisitDate = re.findall('\*DATE\[(.*20\d\d)|\*DATE\[(.*199\d)', MedRecords) #added date
    if len(VisitYear) > 0:
        VisitYear = VisitYear[0][0]
    elif len(VisitYear) == 0:
        VisitYear = 'NaN'
    if len(VisitDate) > 0:
        VisitDate = VisitDate[0][0]
    elif len(VisitDate) == 0:
        VisitDate = 'NaN'
    
    #Getting vitals
    Sub = re.sub('\n', ' ', MedRecords)
    Sub = re.findall('Vitals.*General', Sub)[0]
    BP = re.findall('BP(.*)Weight', Sub)
    Sys = re.findall('(\d+)\/', str(BP)) #take max or sole entry
    Sys = max(Sys)
    Dia = re.findall('\/(\d+)', str(BP)) #take max or sole entry
    Dia = max(Dia)
    BMI = re.findall('BMI(.*)General', Sub) #start point
    BMI = re.findall('\d+\.?\d+', str(BMI))[0] #final tally

    Prep1 = re.split('Sleep Habits: ', MedRecords)
    Prep2 = [re.sub('\\n', ' ', x) for x in Prep1]
    Prep3 = list(chain.from_iterable([re.split('( Time it takes to fall asleep)', x) for x in Prep2]))
    SleepInfo = [x for x in Prep3 if 'Bedtime on weekdays' in x]
    SleepSplit = re.split('[A-Z](?<=[A-Z])', SleepInfo[0])[1:]
    #Generalize
    SleepSplit = [re.sub('(\d+)\: ([1,2,3,4,5][0,5])', '\\1:\\2', x) for x in SleepSplit]
    
    #Change any weekend:10:30pm to add a space and separate the numbers
    SleepSplit = [re.sub('[a-z]\:(\d+)(\:)?(\d+)?', ' \\1\\2\\3', x) for x in SleepSplit]

    #remove extra space
    SleepSplit = [re.sub('(\d+) (\:)(\d+)', ' \\1\\2\\3', x) for x in SleepSplit]
    
    #both 12 and midnight may come up
    for x in range(len(SleepSplit)):
        if bool(re.search('midnight|mdn|mn|midnite|midnt|mignight|mid\-night', str(SleepSplit[x]))) == True and bool(re.search('12', str(SleepSplit[x]))) == True:
            SleepSplit[x] = re.sub('12', '12am', str(SleepSplit[x]))
            SleepSplit[x] = re.sub('midnight|mdn|mn|midnite|midnt', '', str(SleepSplit[x]))
        
    #both 12 and noon may come up
    for x in range(len(SleepSplit)):
        if bool(re.search('noon', str(SleepSplit[x]))) == True and bool(re.search('12', str(SleepSplit[x]))) == True:
            SleepSplit[x] = re.sub('12', '12pm', str(SleepSplit[x]))
            SleepSplit[x] = re.sub('noon', ' ', str(SleepSplit[x]))
            
    Times = [re.sub('[0-9]?[0-9]?midnight|[0-9]?[0-9]?mdn|[0-9]?[0-9]?mn|[0-9]?[0-9]?midnite|[0-9]?[0-9]?midnt|[0-9]?[0-9]?mignight|[0-9]?[0-9]?mid\-night', ' 12am ', x) for x in SleepSplit] #change to "zero" time
    Times = [re.sub(' \:?noon',' 12pm ', x) for x in Times] #change to "zero" time
    Times = [re.sub('(.*)\: ', '', x) for x in Times]
    
    for x in range(len(Times[1:])):
        if 'same' in str(Times[x+1]):
            Times[x+1] = Times[x]
    if len(Times) == 4:
        #if a value is missing from the reported sleep or wake times, but the other value (i.e. weekend but not weekday) is present, then assume it will be the same, or at least similar
        #..because its more likely a doctor will not enter it than ask only 
        if len(re.findall('\d', Times[1])) == 0 and len(re.findall('\d', Times[0])) > 0:
               Times[1] = Times[0]
        if len(re.findall('\d',Times[3]))==0 and len(re.findall('\d', Times[2])) > 0:
               Times[3] = Times[2]
        Times = [re.sub('(\d+) ([a-z])', '\\1\\2', x) for x in Times] #remove space between number and (hopefully) am or pm  
        Times.append(VisitYear) #add year of the visit to calculate the ages later
        Times.append(VisitDate) #add date of the visit to calculate the ages later
        Times.append(Sys) #add vitals
        Times.append(Dia) #add vitals
        Times.append(BMI) #add vitals

        return(Times)


def ChronoDFBuilderSub(WeekdayBed, WeekendBed, WeekdayWake, WeekendWake, GRIDList, Year, Date, Systole, Diastole, BMI):
    TimeEstimates = pd.DataFrame()
    if len(WeekdayBed) == len(WeekendBed) == len(WeekdayWake) == len(WeekendWake) == len(GRIDList):
        for person in range(len(GRIDList)):
            PersonTimes = [WeekdayBed[person], WeekendBed[person], WeekdayWake[person], WeekendWake[person]]
            try:
                TimeConvert = PersonTimeConvert(PersonTimes)
                PersonConvert = list(chain.from_iterable(list([[[GRIDList[person]], TimeConvert, [Year[person]], [Date[person]], [Systole[person]], [Diastole[person]], [BMI[person]]]])))
                PersonConvert = pd.Series(list(chain.from_iterable(PersonConvert)))
                TimeEstimates = TimeEstimates.append(PersonConvert, ignore_index=True)
            except:
                continue
    else:
        print('No match across times, missing data?')
        return
    
    TimeEstimates.columns = ['GRID', 'Weekday Bed Time', 'Weekend Bed Time', 'Weekday Wake Time', 'Weekend Wake Time', 'Year', 'Date', 'Systole', 'Diastole', 'BMI']
    
    return(TimeEstimates)

#A couple "rules" that make sense
#1) Its much more likely if for the bedtime values, if both are double digit, then its pm (unless one is 12, then its likely midnight)
#... its highly unlikely even for a shift worker that they would wake up in double digit hours
#2) Its reasonable to assume that if a pm is assigned to one of the bedtimes, then the other would be pm, unless its single digit
#... for the weekend time, then it could be midnight, 1am, 2am, etc..

def PersonTimeConvert(TimesTest):

    #Default, easiest case is if its specified by am or pm
    TimesTest = [str(x) for x in TimesTest] #changed
    #some entry oddities, like .30 instead of :30
    TimesTest = [re.sub('(\d+\:?[0,1,2,3,4,5]?[0,5]?)to', '\\1', x) for x in TimesTest]
    TimesTest = [re.sub('(\d+\:?[0,1,2,3,4,5]?[0,5]?)or', '\\1', x) for x in TimesTest]
    TimesTest = [re.sub('(\d+)\.([0,1,2,3,4,5]?[0,5]?)', '\\1:\\2', x) for x in TimesTest]
    TimesTest = [re.sub('(\d+)\;([0,1,2,3,4,5]?[0,5]?)', '\\1:\\2', x) for x in TimesTest]
    
    #sometimes three duplicated entires come up.. probably not of much use
    if TimesTest[0] == TimesTest[1] == TimesTest[2] or TimesTest[1] == TimesTest[2] == TimesTest[3]:
        TimesTest = ['NaN', 'NaN', 'NaN', 'NaN']
    
    #I - inference section
    #If 'am' is reported in both wake times, and both sleep times are double digits, its highly likely its 10pm, 11pm, or 12am
    if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', TimesTest[2])) == True and bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', TimesTest[3])) == True and bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimesTest[1])) == False and bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimesTest[0])) == False:
        try:
            if float(re.findall('(\d+)\:?', TimesTest[0])[0]) == 10 or float(re.findall('(\d+)\:?', TimesTest[0])[0]) == 11: #if between 10-12, it should be 
                TimesTest[0] = re.sub('(.*)', '\\1pm', TimesTest[0])
            if float(re.findall('(\d+)\:?', TimesTest[0])[0]) == 12: #if its 12, then make it am, even if its 12:30 
                TimesTest[0] = re.sub('(.*)', '\\1am', TimesTest[0])
            if float(re.findall('(\d+)\:?', TimesTest[1])[0]) == 10 or float(re.findall('(\d+)\:?', TimesTest[1])[0]) == 11: #if between 10-12, it should be 
                TimesTest[1] = re.sub('(.*)', '\\1pm', TimesTest[1])
            if float(re.findall('(\d+)\:?', TimesTest[1])[0]) == 12: #if its 12, then make it am, even if its 12:30 
                TimesTest[1] = re.sub('(.*)', '\\1am', TimesTest[1])
            #New rule - if am is in wake times, then match the weekday wake/sleep and weekend wake/sleep: if sleep value is greater than or equal to wake value
            #.. e.g. 9 and 6am, then assume the person doesn't sleep more than 12 hours? and make it 9pm, or 8pm, etc..
            if float(re.findall('(\d+)\:?', TimesTest[2])[0]) <= float(re.findall('(\d+)\:?', TimesTest[0])[0]): #if between 10-12, it should be 
                TimesTest[0] = re.sub('(.*)', '\\1pm', TimesTest[0])
            if float(re.findall('(\d+)\:?', TimesTest[3])[0]) <= float(re.findall('(\d+)\:?', TimesTest[1])[0]): #if between 10-12, it should be 
                TimesTest[1] = re.sub('(.*)', '\\1pm', TimesTest[1]) 
                
        except:
            pass
    
#Reverse, if pm is in both sleep times, and its 10pm or 11pm (ignore 12am?), then its highly likely that if the wake times are single digit, its am
    if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', TimesTest[2])) == False and bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', TimesTest[3])) == False and bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimesTest[1])) == True and bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimesTest[0])) == True:
        try:
            if float(re.findall('(\d+)\:?', TimesTest[0])[0]) >= 10 and float(re.findall('(\d+)\:?', TimesTest[1])[0]) >= 10 and float(re.findall('(\d+)\:?', TimesTest[2])[0]) < 10 and float(re.findall('(\d+)\:?', TimesTest[3])[0]) < 10: #if between 10-12, it should be 
                TimesTest[2] = re.sub('(.*)', '\\1am', TimesTest[2])
                TimesTest[3] = re.sub('(.*)', '\\1am', TimesTest[3])
        
            #New rule - if pm is in sleep times, then match the weekday wake/sleep and weekend wake/sleep: if wake value is less than or equal to wake value
            #.. e.g. 9 and 6am, then assume the person doesn't sleep more than 12 hours? and make it 9pm, or 8pm, etc..
            if float(re.findall('(\d+)\:?', TimesTest[2])[0]) <= float(re.findall('(\d+)\:?', TimesTest[0])[0]): #if between 10-12, it should be 
                TimesTest[2] = re.sub('(.*)', '\\1am', TimesTest[2])
            if float(re.findall('(\d+)\:?', TimesTest[3])[0]) <= float(re.findall('(\d+)\:?', TimesTest[1])[0]): #if between 10-12, it should be 
                TimesTest[3] = re.sub('(.*)', '\\1am', TimesTest[3]) 
            
        except:
            pass
    
    for x in range(len(TimesTest)):
        #if the second value is 12 or single digit am, the first value is 9 10 or 11, then the first value should have a pm put behind it
        if len(re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))) == 2:
            try:
                TimesTest[x] = re.sub('-', ' - ', str(TimesTest[x]))
                TimeExtracts = re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))
                TimeAndNotationExtractions = [x for x in re.split('(\d+\:?[0,1,2,3,4,5]?[0,5]?\s+)', str(TimesTest[x])) if bool(re.search('\d', x)) == True]
                if len(TimeExtracts) == len(TimeAndNotationExtractions) == 2:
                #if the secnod value contains am and is 12 or less than 10, the first value contains nothing, and the first value is double digit
                    if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', TimeAndNotationExtractions[1])) == True and float(re.findall('(\d+)\:?', TimeExtracts[1])[0]) == 12 or float(re.findall('(\d+)\:?', TimeExtracts[1])[0]) < 10 and bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+ap|\d+pa', TimeAndNotationExtractions[0])) == False and float(re.findall('(\d+)\:?', TimeExtracts[0])[0]) == 10 or float(re.findall('(\d+)\:?', TimeExtracts[0])[0]) == 11: 
                    #Add a pm to that first time value
                        TimesTest[x] = re.sub(re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))[0], re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))[0] + 'pm', TimesTest[x], count = 1)
            
                #Also match the am/pm for two numbers if its not fixed at this point and not speicfied. assume e.g. 1-2am is 1am - 2am
                    elif bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', TimeAndNotationExtractions[1])) == True and bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap|\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimeAndNotationExtractions[0])) == False:
                        TimesTest[x] = re.sub(re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))[0], re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))[0] + 'am', TimesTest[x], count = 1)

                    elif bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimeAndNotationExtractions[1])) == True and bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap|\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', TimeAndNotationExtractions[0])) == False:
                        TimesTest[x] = re.sub(re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))[0], re.findall('(\d+\:?[0,1,2,3,4,5]?[0,5]?)', str(TimesTest[x]))[0] + 'pm', TimesTest[x], count = 1)

            except:
                continue
        #The opposite doesn't hold up as well - its common to see a 12pm entered when it should be am, not common for 12am to accidentally mean noon though

    #Easier to track if separated
    BedValues = TimesTest[0:2]
    WakeValues = TimesTest[2:4]
    
    #If theres an 1130 instead of 11:30, or 930
    for x in range(len(BedValues)):
        DigitLen = re.findall('\d+', BedValues[x])
        if any([len(x) >= 3 for x in DigitLen]):
            BedValues[x] = re.sub('(\d+)([0-9][0-9])', '\\1:\\2', BedValues[x])
        if len(re.findall('\d+', BedValues[x])) == 0:
            BedValues[x] = 'NaN'
            
        #If we can't find an a or p do we assume we can't assign the correct time?
        if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+ap|\d+pa', str(BedValues[x]))) == False:
            BedValues[x] = 'NaN'
            
    #II
    
    TempBed = BedValues      
            
    #if numbers are missing, it'd be dangerous to assume we can infer it from other times
    for x in range(len(TempBed)):
        if len(re.findall('\d+', str(TempBed[x]))) == 0:
            TempBed[x] = 'NaN'
    
    #III
    
    #Whats up with these 10-12pm errors that only seem to happen in bed times.. don't do this for wake times
    for x in range(len(TempBed)):
        if bool(re.search('10\-12p |10\-12pm|10\-12 pm|10\-12  pm|10\-12p$|10\-12p\.|10\-12pa', str(TempBed[x]))) == True:
            TempBed[x] = '10pm - 12am'
        if bool(re.search('11\-12p |11\-12pm|11\-12 pm|11\-12  pm|11\-12p$|11\-12p\.|11\-12pa', str(TempBed[x]))) == True:
            TempBed[x] = '11pm - 12am'
    
    #else:
    for x in range(len(TempBed)):
        #Add in an eg. 10a instead of 10a
        
        #if am and pm show up (in a range of times maybe) - try to get an avg
        if len(re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(TempBed[x]))) == 1:
            TempBed[x] = re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(TempBed[x]))
            
            if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', str(TempBed[x]))) == True:
            #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(TempBed[x]))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(TempBed[x]))[0])/60)), str(TempBed[x]))
                    y = re.sub('\.0', '', y)

                else:
                    y = str(TempBed[x])
                y = re.findall('\d+\.?\d?\d?', y)
                TempBed[x] = float(y[0])
            #If its 12am, set to 0
                if TempBed[x] == 12.0:
                    TempBed[x] = float(0)
                
            if bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', str(TempBed[x]))) == True:
            #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(TempBed[x]))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)' ,str(TempBed[x]))[0])/60)), str(TempBed[x]))
                    y = re.sub('\.0', '', y)

                else:
                    y = str(TempBed[x])
                y = re.findall('\d+\.?\d?\d?', y)
                if '12' not in y[0]:
                    TempBed[x] = float(y[0]) + 12.0
                else:
                    TempBed[x] = float(y[0])
            
        #Getting the average
        if len(re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(TempBed[x]))) > 1:
            
            temp1 = re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(TempBed[x]))[0]
            temp2 = re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(TempBed[x]))[1]

            #Add in an eg. 10a instead of 10a
            if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', str(temp1))) == True:
                #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp1))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp1))[0])/60)), str(temp1))
                    y = re.sub('\.0', '', y)
                else:
                    y = str(temp1)
                y = re.findall('\d+\.?\d?\d?', y)
                temp1 = float(y[0])
                if temp1 == 12.0:
                    temp1 = float(0)
                if temp1 == 12.5:
                    temp1 = float(0.5)
                
            #Add in an eg. 10a instead of 10a
            if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', str(temp2))) == True:
            #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp2))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp2))[0])/60)), str(temp2))
                    y = re.sub('\.0', '', y)
                else:
                    y = str(temp2)
                y = re.findall('\d+\.?\d?\d?', y)
                temp2 = float(y[0])
                if temp2 == 12.0:
                    temp2 = float(0)
                if temp2 == 12.5:
                    temp2 = float(0.5)
            
            if bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', str(temp1))) == True:
                #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp1))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp1))[0])/60)), str(temp1))
                    y = re.sub('\.0', '', y)
                else:
                    y = str(temp1)
                y = re.findall('\d+\.?\d?\d?', y)
                temp1 = float(y[0])
                if temp1 < 12.0:
                    temp1 = temp1 + 12.0

            
            if bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', str(temp2))) == True:
                #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp2))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp2))[0])/60)), str(temp2))
                    y = re.sub('\.0', '', y)

                else:
                    y = str(temp2)
                y = re.findall('\d+\.?\d?\d?', y)
                temp2 = float(y[0])
                if temp2 < 12.0:
                    temp2 = temp2 + 12.0
            
            #in the case of e.g. 10-1am (or 22.0 to 1.0)
            if temp1 > 12.0 and temp2 < 12.0:
                temp2 = temp2 + 24.0
            if temp2 - temp1 > 3.0:
                TempBed[x] = float('NaN')    
            elif temp2 - temp1 <= 3.0:
                Avg = float(np.mean([temp1, temp2]))
                if Avg < 24.0:
                    TempBed[x] = Avg
                elif Avg >= 24.0:
                    TempBed[x] = Avg - 24.0
    

        
    #Wake time - trying to make more general.. same rules may not apply
    for x in range(len(WakeValues)):
        DigitLen = re.findall('\d+', WakeValues[x])
        #Convering e.g. 1130 to 11:30
        if any([len(x) >= 3 for x in DigitLen]):
            WakeValues[x] = re.sub('(\d+)([0-9][0-9])', '\\1:\\2', WakeValues[x])
        if len(re.findall('\d+', WakeValues[x])) == 0:
            WakeValues[x] = 'NaN'
        
        #If we can't find an a or p do we assume we can't assign the correct time?
        if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+ap|\d+pa', str(WakeValues[x]))) == False:
            WakeValues[x] = 'NaN'

        if len(re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(WakeValues[x]))) == 1:
            WakeValues[x] = re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(WakeValues[x]))
            
            if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', str(WakeValues[x]))) == True:
            #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(WakeValues[x]))) >0 :
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(WakeValues[x]))[0])/60)), str(WakeValues[x]))
                    y = re.sub('\.0', '', y)

                else:
                    y = str(WakeValues[x])
                y = re.findall('\d+\.?\d?\d?', y)
                WakeValues[x] = float(y[0])
            #If its 12am, set to 0
                if WakeValues[x] == 12.0:
                    WakeValues[x] = float(0)
                
            if bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', str(WakeValues[x]))) == True:
            #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)',str(WakeValues[x]))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(WakeValues[x]))[0])/60)), str(WakeValues[x]))
                    y = re.sub('\.0', '', y)

                else:
                    y = str(WakeValues[x])
                y = re.findall('\d+\.?\d?\d?', y)
                if '12' not in y[0]:
                    WakeValues[x] = float(y[0]) + 12.0
                else:
                    WakeValues[x] = float(y[0])
            
        #Getting the average
        if len(re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(WakeValues[x]))) > 1:
            
            temp1 = re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(WakeValues[x]))[0]
            temp2 = re.findall('[0-9]?[0-9]?\:?\d+p |[0-9]?[0-9]?\:?\d+pm|[0-9]?[0-9]?\:?\d+ pm|[0-9]?[0-9]?\:?\d+  pm|[0-9]?[0-9]?\:?\d+p$|[0-9]?[0-9]?\:?\d+p\.|[0-9]?[0-9]?\:?\d+a |[0-9]?[0-9]?\:?\d+am|[0-9]?[0-9]?\:?\d+ am|[0-9]?[0-9]?\:?\d+  am|[0-9]?[0-9]?\:?\d+a$|[0-9]?[0-9]?\:?\d+a\.|[0-9]?[0-9]?\:?\d+ap|[0-9]?[0-9]?\:?\d+pa', str(WakeValues[x]))[1]

            #Add in an eg. 10a instead of 10a
            if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', str(temp1))) == True:
                #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp1))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp1))[0])/60)), str(temp1))
                    y = re.sub('\.0', '', y)
                else:
                    y = str(temp1)
                y = re.findall('\d+\.?\d?\d?', y)
                temp1 = float(y[0])
                if temp1 == 12.0:
                    temp1 = float(0)
                if temp1 == 12.5:
                    temp1 = float(0.5)
                
            #Add in an eg. 10a instead of 10a
            if bool(re.search('\d+a |\d+am|\d+ am|\d+  am|\d+a$|\d+a\.|\d+ap', str(temp2))) == True:
            #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp2))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp2))[0])/60)), str(temp2))
                    y = re.sub('\.0', '', y)
                else:
                    y = str(temp2)
                y = re.findall('\d+\.?\d?\d?', y)
                temp2 = float(y[0])
                if temp2 == 12.0:
                    temp2 = float(0)
                if temp2 == 12.5:
                    temp2 = float(0.5)
            
            if bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', str(temp1))) == True:
                #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp1))) > 0:
                    y = re.sub('\:(\d+)', '.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp1))[0])/60)), str(temp1))
                    y = re.sub('\.0', '', y)
                else:
                    y = str(temp1)
                y = re.findall('\d+\.?\d?\d?', y)
                temp1 = float(y[0])
                if temp1 < 12.0:
                    temp1 = temp1 + 12.0

            
            if bool(re.search('\d+p |\d+pm|\d+ pm|\d+  pm|\d+p$|\d+p\.|\d+pa', str(temp2))) == True:
                #Generalize the conversion from :15 to 0.25 for any minute
                if len(re.findall('\:(\d+)', str(temp2))) > 0:
                    y = re.sub('\:(\d+)','.{0}'.format(str(float(re.findall('[0-9]?[0-9]\:(\d+)', str(temp2))[0])/60)), str(temp2))
                    y = re.sub('\.0', '', y)

                else:
                    y = str(temp2)
                y = re.findall('\d+\.?\d?\d?', y)
                temp2 = float(y[0])
                if temp2 < 12.0:
                    temp2 = temp2 + 12.0
            
            #in the case of e.g. 10-1am (or 22.0 to 1.0)
            if temp1 > 12.0 and temp2 < 12.0:
                temp2 = temp2 + 24.0
            if temp2 - temp1 > 3.0:
                WakeValues[x] = float('NaN')    
            elif temp2 - temp1 <= 3.0:
                Avg = float(np.mean([temp1, temp2]))
                if Avg < 24.0:
                    WakeValues[x] = Avg
                elif Avg >= 24.0:
                    WakeValues[x] = Avg - 24.0
            
    TempBed = [re.sub('24.0', '0.0', str(x)) for x in TempBed]
    TempBed = [float(x) for x in TempBed]    
    WakeValues = [re.sub('24.0', '0.0', str(x)) for x in WakeValues]
    WakeValues = [float(x) for x in WakeValues]
    Output = list(chain.from_iterable(list([TempBed, WakeValues])))
    return(Output)


#Keep the GRIDList referenced outside the function, to save memory and not recreate the ClinDocs list 16x for the mp.pooler()
def ChronoDFBuilderOverall2(SubClinDocs): #Maybe we can pull out chronotype; subclindocs refers to the split of the clinical docs
    WeekdayBedTime = []
    WeekendBedTime = []
    WeekdayWakeTime = []
    WeekendWakeTime = []
    GRIDTracker = []
    DateTracker = []
    Date2Tracker = []
    SysTracker = []
    DiaTracker = []
    BMITracker = []
    
    #GRID list from within
    SubGRIDList = list(set(SubClinDocs.GRID))
    
    for grid in SubGRIDList:
        tempdoc = [x for x in SubClinDocs.CONTENT[(SubClinDocs.DOC_TYPE=='HP') & (SubClinDocs.GRID==grid)].values if 'Sleep Habits: ' in str(x)]
        if len(tempdoc) > 0:
            for Visit in tempdoc:
                try:
                    SleepTime = ChronoEstimator(Visit)
                    if SleepTime is not None:
                        GRIDTracker.append(grid)
                        WeekdayBedTime.append(SleepTime[0])
                        WeekendBedTime.append(SleepTime[1])
                        WeekdayWakeTime.append(SleepTime[2])
                        WeekendWakeTime.append(SleepTime[3])
                        DateTracker.append(SleepTime[4])
                        Date2Tracker.append(SleepTime[5]) #added the specific date
                        SysTracker.append(SleepTime[6]) #added the specific date
                        DiaTracker.append(SleepTime[7]) #added the specific date
                        BMITracker.append(SleepTime[8]) #added the specific date


                except:
                    continue
                    
    try:
        OutputDF = ChronoDFBuilderSub(WeekdayBedTime, WeekendBedTime, WeekdayWakeTime, WeekendWakeTime, GRIDTracker, DateTracker, Date2Tracker, SysTracker, DiaTracker, BMITracker)
    except:
        return
    if OutputDF is None:
        return(None)
    else:
        return(OutputDF)

#pooler function - chunking and multi-core use
OutputDF = pd.DataFrame()

for ClinDocs in pd.read_csv('ClinDocs', chunksize = 200000):
#    
    DocSplit = list(np.array_split(ClinDocs, 16)) #break into chunks
    
    pooler = mp.Pool(16)
    for result in pooler.imap(ChronoDFBuilderOverall2, DocSplit):
        if result is not None:
            OutputDF = OutputDF.append(result, ignore_index=True)

    pooler.close()
    pooler.join()
    gc.collect()

OutputDF.to_csv('SleepDataRaw.csv')
