#Calculating circular stats and social lags
#Input: SleepDataTrimMetrics.csv
#Output: SleepDataClean.csv
library(readr)
library(dplyr)
library(tidyr)
library(ggplot2)
library(ggthemes)
library(circular)
options(warn = -1)

#filepath = file.path('..')
filepath = getwd()

#Read in sleep phenotypes
Chrono = read_csv(file = file.path(filepath, 'SleepDataTrimMetrics.csv'))
colnames(Chrono) = gsub(' ', '.', colnames(Chrono))

circDiff = function(x, y, maxVal=24) {
  x = x / maxVal
  y = y / maxVal
  if (is.vector(y)) {
    d1 = unname(y - x)
    d2 = cbind(d1, d1-1, d1+1)
    d3 = apply(d2, MARGIN=1, function(d) d[which.min(abs(d))])
    d4 = sapply(d3, function(d) ifelse(length(d)==0, NA, d))
  } else {
    d4 = y
    for (jj in 1:ncol(y)) {
      d1 = unname(y[,jj] - x)
      d2 = cbind(d1, d1-1, d1+1)
      d3 = apply(d2, MARGIN=1, function(d) d[which.min(abs(d))])
      d4[,jj] = sapply(d3, function(d) ifelse(length(d)==0, NA, d))}}
  return(d4 * maxVal)}


#Calculate the social lags
Chrono$Social.Lag.Bed = circDiff(Chrono$Weekday.Bed.Time, Chrono$Weekend.Bed.Time)
Chrono$Social.Lag.Wake = circDiff(Chrono$Weekday.Wake.Time, Chrono$Weekend.Wake.Time)
Chrono$Social.Lag.Midpoint = circDiff(Chrono$Weekday.Sleep.Midpoint, Chrono$Weekend.Sleep.Midpoint)

Chrono$Weekday.Bed.TimeAdj = circDiff(Chrono$Weekday.Bed.Time, mean.circular(as.circular(Chrono$Weekday.Bed.Time, units = 'hours'), na.rm = TRUE)[[1]])
Chrono$Weekday.Wake.TimeAdj = circDiff(Chrono$Weekday.Wake.Time, mean.circular(as.circular(Chrono$Weekday.Wake.Time, units = 'hours'), na.rm = TRUE)[[1]])
Chrono$Weekend.Bed.TimeAdj = circDiff(Chrono$Weekend.Bed.Time, mean.circular(as.circular(Chrono$Weekend.Bed.Time, units = 'hours'), na.rm = TRUE)[[1]])
Chrono$Weekend.Wake.TimeAdj = circDiff(Chrono$Weekend.Wake.Time, mean.circular(as.circular(Chrono$Weekend.Wake.Time, units = 'hours'), na.rm = TRUE)[[1]])
Chrono$Weekday.Sleep.MidpointAdj = circDiff(Chrono$Weekday.Sleep.Midpoint, mean.circular(as.circular(Chrono$Weekday.Sleep.Midpoint, units = 'hours'), na.rm = TRUE)[[1]])
Chrono$Weekend.Sleep.MidpointAdj = circDiff(Chrono$Weekend.Sleep.Midpoint, mean.circular(as.circular(Chrono$Weekend.Sleep.Midpoint, units = 'hours'), na.rm = TRUE)[[1]])

#Write to file so the CalculateSleepMetrics.ipynb can calculate sleep metrics
write_csv(Chrono, path = file.path(filepath, 'SleepDataClean.csv'))
