#Figure plotting sleep time distribution - used as Figure 1
#Input: SleepDataClean.csv
#Output: Fig1.pdf

library(ggplot2)
library(readr)

#filepath = file.path('..')
filepath = getwd()
dir.create(path = file.path(filepath, 'CohortMatching_Descriptives'))


#Read in sleep phenotypes
Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))

#Adjust bed times for plotting purposes
Chrono$Weekday.Bed.Time[!is.na(Chrono$Weekday.Bed.Time) & Chrono$Weekday.Bed.Time < 12] = Chrono$Weekday.Bed.Time[!is.na(Chrono$Weekday.Bed.Time) & Chrono$Weekday.Bed.Time < 12] + 24
Chrono$Weekend.Bed.Time[!is.na(Chrono$Weekend.Bed.Time) & Chrono$Weekend.Bed.Time < 12] = Chrono$Weekend.Bed.Time[!is.na(Chrono$Weekend.Bed.Time) & Chrono$Weekend.Bed.Time < 12] + 24

SleepTimeGG = as.data.frame(rbind(cbind(c(rep('Weekday Bed', nrow(Chrono))), Chrono$Weekday.Bed.Time),
cbind(c(rep('Weekend Bed', nrow(Chrono))), Chrono$Weekend.Bed.Time),
cbind(c(rep('Weekday Wake', nrow(Chrono))), Chrono$Weekday.Wake.Time),
cbind(c(rep('Weekend Wake', nrow(Chrono))), Chrono$Weekend.Wake.Time)))

colnames(SleepTimeGG) = c('TimeGroup', 'Value')
SleepTimeGG$Value = as.numeric(as.character(unlist(SleepTimeGG$Value)))

safecolors = c('#d7191c', '#fdae61', '#abd9e9', '#2c7bb6')

NoSleepRadialPlot = ggplot(SleepTimeGG)+
  theme_bw()+
  ylab('Density')+
  xlab('Clock time')+
  scale_fill_manual(values = safecolors)+
  scale_x_continuous(breaks = seq(0, 36, by = 4), labels = c('00:00', '04:00', '08:00', '12:00', '16:00', '20:00', '00:00', '04:00', '08:00', '12:00'), expand = c(0, 0), lim = c(0, 36))+
  geom_density(aes(x=Value, fill=TimeGroup), alpha = 0.5, adjust = 5/4)+
  theme(axis.text.y = element_text(size = 11), axis.title = element_text(size = 12, face = "bold"), 
        legend.title = element_blank(), legend.text = element_text(size = 10.5), legend.key.size = unit(1, "line"),
        axis.text.x = element_text(size = 10))

ggsave(filename = "Fig1.pdf", plot = NoSleepRadialPlot, path = file.path(filepath, 'CohortMatching_Descriptives'), width = 6, height = 4)
