#Trim SNPs to sleep cohort from the Vanderbilt Exome BeadChip data
#Inputs: SleepDataClean.csv, ExomePubSnpTrim.rlist, MegaPubSnpTrim.rlist
#Outputs: TrimmedGRIDSnpsMega.csv, FinalGridSnpMatMega.csv, TrimmedGRIDSnpsExome.csv, FinalGridSnpMatExome.csv
import pandas as pd
import numpy as np
import re, csv
import warnings
from itertools import chain
warnings.filterwarnings('once')

Chrono = pd.read_csv('SleepDataClean.csv')

GRIDs = list(set(Chrono.GRID))

def GRIDMatch(SnpMat):
    return(list(set(re.split('\ ',SnpMat[0])).intersection(GRIDs)))

#Read in large SNP file line by line, match to cohort list
with open('MegaPubSnpTrim.rlist') as csvfile:
    for line in csvfile:
        GridSnps = pd.Series([line], dtype = str)
        GridSnps.values[0] = re.sub('\\n', '', GridSnps.values[0])
        Matches = GRIDMatch(GridSnps) #one row
        Matches = pd.DataFrame([Matches])
        temp = re.split('\ ', GridSnps[0])
        SnpNames = pd.Series([temp[0] + '_' + temp[1]])
        SnpNames = pd.DataFrame(SnpNames)
        Combo = SnpNames.merge(Matches, left_index = True, right_index = True)
        with open('GWAS/TrimmedGRIDSnpsMega.csv', 'a') as fp:
            Combo.to_csv(fp,index = False, header = False)

Snps = pd.read_csv('GWAS/TrimmedGRIDSnpsMega.csv', header = None, sep = ',', names = np.arange(0, 1000), dtype = str)

UniSnps = [re.sub('\_.*', '', x) for x in Snps[Snps.columns[0]].values]
UniSnps = list(set(UniSnps))

Names = ['GRID'] + UniSnps

#initialize to NA.. since there is no
GridSnpMat = pd.DataFrame(np.zeros((len(GRIDs), len(Names))))
GridSnpMat.columns = Names
GridSnpMat.GRID = GRIDs

for row in range(len(Snps)):
    ColName = re.sub('_.*', '', Snps[Snps.columns[0]][row])
    GridList = [re.findall('R.*', str(x)) for x in Snps.iloc[row].values]
    GridList = list(chain.from_iterable(GridList))
    GridList = [x for x in GridList if len(x) > 0]
    #Find Hets, assign 1 to that snp column
    if bool(re.findall('HET', Snps[Snps.columns[0]][row])) == True:
        GridSnpMat[ColName][GridSnpMat.GRID.isin(GridList)] = 1 

    elif bool(re.findall('HOM',Snps[Snps.columns[0]][row])) == True:
        GridSnpMat[ColName][GridSnpMat.GRID.isin(GridList)] = 2 

    elif bool(re.findall('NIL',Snps[Snps.columns[0]][row])) == True:
        GridSnpMat[ColName][GridSnpMat.GRID.isin(GridList)] = 'NaN'

GridSnpMat.to_csv('GWAS/FinalGridSnpMatMega.csv')

#Read in large SNP file line by line, match to cohort list
with open('ExomePubSnpTrim.rlist') as csvfile:
    for line in csvfile:
        GridSnps = pd.Series([line], dtype = str)
        GridSnps.values[0] = re.sub('\\n', '', GridSnps.values[0])
        Matches = GRIDMatch(GridSnps) #one row
        Matches = pd.DataFrame([Matches])
        temp = re.split('\ ', GridSnps[0])
        SnpNames = pd.Series([temp[0] + '_' + temp[1]])
        SnpNames = pd.DataFrame(SnpNames)
        Combo = SnpNames.merge(Matches, left_index = True, right_index = True)
        with open('GWAS/TrimmedGRIDSnpsExome.csv', 'a') as fp:
            Combo.to_csv(fp,index = False, header = False)

Snps = pd.read_csv('GWAS/TrimmedGRIDSnpsExome.csv', header = None, sep = ',', names = np.arange(0, 1000), dtype = str)

UniSnps = [re.sub('\_.*', '', x) for x in Snps[Snps.columns[0]].values]
UniSnps = list(set(UniSnps))

Names = ['GRID'] + UniSnps

#initialize to NA.. since there is no
GridSnpMat = pd.DataFrame(np.zeros((len(GRIDs), len(Names))))
GridSnpMat.columns = Names
GridSnpMat.GRID = GRIDs

for row in range(len(Snps)):
    ColName = re.sub('_.*', '', Snps[Snps.columns[0]][row])
    GridList = [re.findall('R.*', str(x)) for x in Snps.iloc[row].values]
    GridList = list(chain.from_iterable(GridList))
    GridList = [x for x in GridList if len(x) > 0]
    #Find Hets, assign 1 to that snp column
    if bool(re.findall('HET', Snps[Snps.columns[0]][row])) == True:
        GridSnpMat[ColName][GridSnpMat.GRID.isin(GridList)] = 1 

    elif bool(re.findall('HOM',Snps[Snps.columns[0]][row])) == True:
        GridSnpMat[ColName][GridSnpMat.GRID.isin(GridList)] = 2 

    elif bool(re.findall('NIL',Snps[Snps.columns[0]][row])) == True:
        GridSnpMat[ColName][GridSnpMat.GRID.isin(GridList)] = 'NaN'

GridSnpMat.to_csv('GWAS/FinalGridSnpMatExome.csv')
