#Melt list of SNPs for PLINK search
#Inputs: PublishedChronoSNPsProxies.csv, Mega_IDs.csv, ExomeIDs.csv
#Mega_IDs.csv is generated from the emerge server (select SNP, rsID from summary_ref_alleles)
#ExomeIDs.csv is generated from the emerge server (select * from Exome_IDs)
#Output: FlatPubMegaSNPs.csv, FlatPubExomeSNPs.csv, MegaPubSnpTrim.rlist (plink), ExomePubSnpTrim.rlist (plink)

library(readr)

filepath = getwd()
dir.create(path = file.path(filepath, 'GWAS'))

#Published SNPs
PubSNPs = read_csv(file = file.path(filepath, 'PublishedChronoSNPsProxies.csv'))

#Mega IDs
IDs = read_csv(file = file.path(filepath, "Mega_IDs.csv"))
#Concat all entries from the table
PubSNPs = melt(PubSNPs, id.vars = 'Mega')$value
PubSNPs = unique(PubSNPs[grep('rs', PubSNPs)])

#Convert to exm
PubSNPsExm = IDs$SNP[IDs$rsID %in% PubSNPs]
PubSNPsExm = as.data.frame(PubSNPsExm)
colnames(PubSNPsExm) = c("SNP")

write_csv(PubSNPsExm, path = file.path(filepath, 'GWAS', 'FlatPubMegaSNPs.csv'))

#Exome IDs
IDs = read_csv(file = file.path(filepath, "ExomeIDs.csv"))
#Concat all entries from the table
PubSNPs = melt(PubSNPs, id.vars = 'Mega')$value
PubSNPs = unique(PubSNPs[grep('rs', PubSNPs)])

#Convert to exm
PubSNPsExm = IDs$SNP[IDs$rsID %in% PubSNPs]
PubSNPsExm = as.data.frame(PubSNPsExm)
colnames(PubSNPsExm) = c("SNP")

write_csv(PubSNPsExm, path = file.path(filepath, 'FlatPubExomeSNPs.csv'))

#These files are used in PLINK to generate rlists used for the targeted GWAS
#plink --bfile ../MEGA2_Euro --extract FlatPubMegaSNPs.csv --recode-rlist --out MegaPubSnpTrim
#plink --bfile ../Exome_36k_cleano_Ws_GRID --extract FlatPubExomeSNPs.csv --recode-rlist --out ExomePubSnpTrim

