#Borrowing from https://genome.sph.umich.edu/wiki/Power_Calculations:_Quantitative_Traits
#.. and Derkach et al.'s PAGEANT paper

library(readr)
filepath = getwd()

sink(file.path(filepath, 'GWAS', 'PowerAnalysis.txt'))
alpha = 0.05
N = 555 #number of individuals with genotyping

#rs12140153 - Jones et al
MAF = 0.9 #MAF of another "effect" allele from the paper
Beta2 = 0.039
threshold = qchisq(alpha, df = 1, lower.tail = FALSE) #threshold for chisq statistic
powerMAF = pchisq(threshold, df = 1, lower.tail = FALSE, ncp = N * Beta2 * 2*MAF *(1-MAF)) #using Derkach et al. ncp
cat('Power for rs12140153:', powerMAF, '\n') # = 0.506

#rs1144566 - Lane et al
MAF = 0.97 #MAF of another "effect" allele from the paper
Beta2 = 0.099
threshold = qchisq(alpha, df = 1, lower.tail = FALSE) #threshold for chisq statistic
powerMAF = pchisq(threshold, df = 1, lower.tail = FALSE, ncp = N * Beta2 * 2*MAF *(1-MAF)) #using Derkach et al. ncp
cat('Power for rs1144566:', powerMAF, '\n') # = 0.431

#rs35333999 - Lane et al
MAF = 0.043 #MAF of another "effect" allele from the paper
Beta2 = 0.059
threshold = qchisq(alpha, df = 1, lower.tail = FALSE) #threshold for chisq statistic
powerMAF = pchisq(threshold, df = 1, lower.tail = FALSE, ncp = N * Beta2 * 2*MAF *(1-MAF)) #using Derkach et al. ncp
cat('Power for rs35333999:', powerMAF, '\n') # = 0.375

sink()

#Not on our platform but in LD with our "hit"
#rs10157197 - Jones et al
#MAF = 0.6 #MAF of another "effect" allele from the paper
#Beta2 = 0.025
#threshold = qchisq(alpha, df = 1, lower.tail = FALSE) #threshold for chisq statistic
#powerMAF = pchisq(threshold, df = 1, lower.tail = FALSE, ncp = N * Beta2 * 2*MAF *(1-MAF)) #using Derkach et al. ncp
#powerMAF # = 0.73
