#Demographic comparisons in sleep data
#Inputs: SleepDataClean.csv
#Outputs: UnivariateAnalysis_Routputs.txt
library(readr)
library(circular)
options(warn = -1)

#filepath = file.path('..')
filepath = getwd()

#Read in sleep phenotypes
Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))

#############
#Below are basic univariate tests used in the manuscript - sunk to .txt file
#############

sink(file.path(filepath, "CohortMatching_Descriptives", "UnivariateAnalysis_Routputs.txt"))

#Note the demographic breakdowns need to be done by UNIQUE individuals, since some people have multiple entries
#The BMI for individuals with multiple entries can stay though, since it can vary over time
print("Gender breakdown")
paste0("Male: ", nrow(subset(Chrono, !duplicated(Chrono$GRID) & GENDER_EPIC=="M"))/nrow(subset(Chrono, !duplicated(Chrono$GRID))),
       "Female: ", nrow(subset(Chrono, !duplicated(Chrono$GRID) & GENDER_EPIC=="F"))/nrow(subset(Chrono, !duplicated(Chrono$GRID))))
cat('\n', '\n')

print("Race breakdown")
paste0("White: ", nrow(subset(Chrono, !duplicated(Chrono$GRID) & SD_RACE=="W"))/nrow(subset(Chrono, !duplicated(Chrono$GRID))),
       " Black: ", nrow(subset(Chrono, !duplicated(Chrono$GRID) & SD_RACE=="B"))/nrow(subset(Chrono, !duplicated(Chrono$GRID))),
      " Other: ", nrow(subset(Chrono, !duplicated(Chrono$GRID) & SD_RACE=="O"))/nrow(subset(Chrono, !duplicated(Chrono$GRID))))
cat('\n', '\n')

print("Age at visit")
paste0("Mean: ", mean(Chrono$VisitAge, na.rm = TRUE), " ", "SD: ", sd(Chrono$VisitAge, na.rm = TRUE))
cat('\n', '\n')

print("Weekday Wake Time")
weekdaywake = mean.circular(as.circular(Chrono$Weekday.Wake.Time, units = 'hours'), na.rm = TRUE)[[1]]
paste0(as.numeric(substr(weekdaywake, 1, 1)), ":",
       round(as.numeric(substr(weekdaywake, 2, 5)) * 60))
cat('\n', '\n')

print("Weekend Wake Time")
weekendwake = mean.circular(as.circular(Chrono$Weekend.Wake.Time, units = 'hours'), na.rm = TRUE)[[1]]
paste0(as.numeric(substr(weekendwake, 1, 1)), ":",
       round(as.numeric(substr(weekendwake, 2, 5)) * 60))
cat('\n', '\n')

print("Weekday to Weekend Wake Time")
t.test(Chrono$Weekday.Wake.Time,Chrono$Weekend.Wake.Time)
paste("Actual p-value:", t.test(Chrono$Weekday.Wake.Time,Chrono$Weekend.Wake.Time)$p.value)
cat('\n', '\n')

print("Weekday to Weekend Sleep Duration")
t.test(Chrono$Weekday.Sleep.Duration,Chrono$Weekend.Sleep.Duration)
paste("Actual p-value:", t.test(Chrono$Weekday.Sleep.Duration,Chrono$Weekend.Sleep.Duration)$p.value)
cat('\n', '\n')

print("Weekend Sleep Duration - Gender")
t.test(Chrono$Weekend.Sleep.Duration[Chrono$GENDER_EPIC == 'M'], Chrono$Weekend.Sleep.Duration[Chrono$GENDER_EPIC == 'F'])
cat('\n', '\n')

print("Weekend Sleep Duration - Race")
t.test(Chrono$Weekend.Sleep.Duration[Chrono$SD_RACE == 'W'], Chrono$Weekend.Sleep.Duration[Chrono$SD_RACE == 'B'], var.equal = FALSE)
cat('\n', '\n')

print("BMI - Cohort")
paste0("Mean: ", mean(Chrono$BMI, na.rm = TRUE), " SD: ", sd(Chrono$BMI, na.rm = TRUE))

#Check midpoint
print("Weekday Sleep Midpoint")
weekdaymid = mean.circular(as.circular(Chrono$Weekday.Sleep.Midpoint, units = 'hours'), na.rm = TRUE)[[1]]
paste0(as.numeric(substr(weekdaymid, 1, 1)), ":",
       round(as.numeric(substr(weekdaymid, 2, 5)) * 60))
cat('\n', '\n')

print("Weekend Sleep Midpoint")
weekendmid = mean.circular(as.circular(Chrono$Weekend.Sleep.Midpoint, units = 'hours'), na.rm = TRUE)[[1]]
paste0(as.numeric(substr(weekendmid, 1, 1)), ":0",
       round(as.numeric(substr(weekendmid, 2, 5)) * 60))
cat('\n', '\n')

print("Weekday vs. Weekend Midpoint")
t.test(Chrono$Weekday.Sleep.Midpoint, Chrono$Weekend.Sleep.Midpoint)
paste("Actual p-value:", t.test(Chrono$Weekday.Sleep.Midpoint, Chrono$Weekend.Sleep.Midpoint)$p.value)

sink()
