#Cleaning up the raw results from the clinical note parser, after deciding on manual edits
#Input: SleepDataRaw.csv, GRIDDemographics.csv, GRIDDrops.csv, GRIDChangeLog.csv
#Output: SleepDataTrim.csv
library(readr)
library(dplyr)
library(tidyr)
library(ggplot2)
library(ggthemes)

#filepath = file.path('..')
filepath = getwd()

Chrono = read_csv(file = file.path(filepath, 'SleepDataRaw.csv'))
Drops = read_csv(file = file.path(filepath, 'GRIDDrops.csv')) #Dropping due to vague clinical entry
Edits = read_csv(file = file.path(filepath, 'GRIDChangeLog.csv')) #Manual curations
Demos = read_csv(file.path(filepath, 'GRIDDemographics.csv'))


#Replace with manual edits
Chrono = Chrono[!c(Chrono$GRID %in% Drops$GRID), ]
#Match by GRID, Year, Date
for(Ed in 1:nrow(Edits)){
  Chrono[(Chrono$GRID==Edits$GRID[Ed] & Chrono$Year==Edits$Year[Ed] & Chrono$Date==Edits$Date[Ed]),]['Weekday Bed Time'] = Edits$`Weekday Bed Time`[Ed]
  Chrono[(Chrono$GRID==Edits$GRID[Ed] & Chrono$Year==Edits$Year[Ed] & Chrono$Date==Edits$Date[Ed]),]['Weekday Wake Time'] = Edits$`Weekday Wake Time`[Ed]
  Chrono[(Chrono$GRID==Edits$GRID[Ed] & Chrono$Year==Edits$Year[Ed] & Chrono$Date==Edits$Date[Ed]),]['Weekend Bed Time'] = Edits$`Weekend Bed Time`[Ed]
  Chrono[(Chrono$GRID==Edits$GRID[Ed] & Chrono$Year==Edits$Year[Ed] & Chrono$Date==Edits$Date[Ed]),]['Weekend Wake Time'] = Edits$`Weekend Wake Time`[Ed]
}

colnames(Chrono) = gsub(' ', '.', colnames(Chrono))

#Remove rows with multiple visits to the sleep clinic, but duplicated entries in the notes
Chrono = distinct(Chrono, GRID, Weekday.Wake.Time, Weekend.Wake.Time, Weekday.Bed.Time, Weekend.Bed.Time, .keep_all = TRUE)

#Drop rows where an NA exists for all 4 variables
Chrono = Chrono[-which(is.na(Chrono$Weekday.Bed.Time) & is.na(Chrono$Weekday.Wake.Time) & is.na(Chrono$Weekend.Bed.Time) & is.na(Chrono$Weekend.Wake.Time)), ]

#Match
ChronoMatch = merge(Chrono, Demos, by = 'GRID')

#Get age at time of visit, then drop missing entries (some people don't have a year or DOB entered)
ChronoMatch$VisitAge = ChronoMatch$Year - as.numeric(substr(ChronoMatch$DOB, start = 1, stop = 4))
ChronoMatch = drop_na(ChronoMatch, VisitAge)

#Adults
ChronoMatch = ChronoMatch[ChronoMatch$VisitAge >= 18, ]

#Keep only white, black and unspecified
#ChronoMatch = ChronoMatch[grep('A|I|N', ChronoMatch$SD_RACE, invert = TRUE), ]
#Remove NA entries
ChronoMatch = ChronoMatch[!is.na(ChronoMatch$SD_RACE), ]
ChronoMatch$SD_RACE = as.character(unlist(ChronoMatch$SD_RACE))

#Set all non-white and black to "O" (other)
ChronoMatch$SD_RACE[grep('W|B', ChronoMatch$SD_RACE, invert = TRUE)] = 'O'
ChronoMatch$SD_RACE = as.factor(ChronoMatch$SD_RACE)

#Some extreme BMIs, wrongly entered perhaps in the records
ChronoMatch$BMI[ChronoMatch$BMI > 80] = NA
ChronoMatch$BMI[ChronoMatch$BMI < 15] = NA

#Write to file so the CalculateSleepMetrics.ipynb can calculate sleep metrics
write_csv(ChronoMatch, path = file.path(filepath, 'SleepDataTrim.csv'))
